/*  @(#)80  1.3.1.1 src/pdwebars/webapp/WEB-INF/classes/amwebarsentitlementservice/protocol/exampleProtocol.java, pdweb.dynadi, amweb510, 031011a 03/09/25 13:11:02  */

package amwebarsentitlementservice.protocol;

import java.util.Iterator;
import java.util.List;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import org.jdom.Element;
import org.jdom.Document;
import org.jdom.JDOMException;

import sun.io.MalformedInputException;

import amwebarsentitlementservice.container.AMWebARSContainer;
import amwebarsentitlementservice.container.AMWebARSContainerDescriptor;
import amwebarsentitlementservice.protocol.status.*;
import amwebarsentitlementservice.support.SharedSaxBuilder;


/**
 * @author Brook Lovatt (blovatt#us.ibm.com)
 * @version 1.0
 */
public class exampleProtocol extends FixedProviderProtocol {

private final String exampleProtocol_java_sourceCodeID="$Id: @(#)80  1.3.1.1 src/pdwebars/webapp/WEB-INF/classes/amwebarsentitlementservice/protocol/exampleProtocol.java, pdweb.dynadi, amweb510, 031011a 03/09/25 13:11:02 $";
	
	// CONSTANTS
	private final static String TEST_CONTAINER_ID = "example_protocol_test_container";
	private final static String XML_TEST_DATA_FILE = "exampleContainer.xml";
	
	// VARIABLES
	private static Document _xml_test_data = null;
	
	
	// CONSTRUCTORS
	/**
	 * Constructor for exampleProtocol.
	 * @param protocol_id
	 */
	public exampleProtocol(String protocol_id) {
		super(protocol_id);
	}
	
	// METHODS
	/**
	 * @see amwebarsentitlementservice.protocol.AMWebARSProtocol#initialize()
	 */
	public ProtocolInitStatus initialize() {
		/* ***************************************************** *
		 * BEGIN: Protocol specific implementation...
		 * 
		 * This may involve establishing a connection to a Third 
		 * Party Profiling Service, or initializing a local database.
		 * 
		 * For this example, we just read in a test container from 
		 * a file called exampleContainer.xml  
		 * ***************************************************** *
		 */
			
		try {
			// Stream in the data from the file
			BufferedInputStream in_stream = new BufferedInputStream(new FileInputStream(XML_TEST_DATA_FILE));
			
			// Build a org.jdom.Document from the data
			_xml_test_data = SharedSaxBuilder.getInstance().build(in_stream);
			
			// Close the file
			in_stream.close();
			
			// If no exceptions have been thrown yet, then we can safely say
			// that this protocol has been successfully initialized
			_initialized = true;
			return PROTOCOL_INIT_SUCCESS;
		
		
		// Deal with Exceptions by inserting them into Protocol Exceptions that 
		// will store the Exception data until we've returned to a higher level 
		// of processing the initialize request.  At that point the exceptions 
		// will be inserted in the response to the initialize request.	
		} catch (FileNotFoundException file_ex) {
			file_ex.printStackTrace();
			super._initialized = false;
			return new ProtocolInitException(file_ex);
		} catch (IOException io_ex) {
			io_ex.printStackTrace();
			super._initialized = false;
			return new ProtocolInitException(io_ex);
		} catch (JDOMException jdom_ex) {
			jdom_ex.printStackTrace();
			super._initialized = false;
			return new ProtocolInitException(jdom_ex);	
		} 
		
		/* ***************************************************** *
		 * END: Protocol specific implementation...
		 * ***************************************************** *
		 */
	}

	
	/**
	 * @see amwebarsentitlementservice.protocol.AMWebARSProtocol#run()
	 */
	public ProtocolRunStatus run() {
		
		if (!isInitialized()) {
			// This protocol has not been initialized
			return new ProtocolRunException(new UnsupportedOperationException("Protocol not initialized correctly."));	
		}
		
		// Get an iterator to the hashtable of container descriptors that must be retrieved.
		Iterator container_descriptors = getClient().elements();
		
		// Iterate through the container descriptors.
		while (container_descriptors.hasNext()) {
			
			// Get the next container descriptor: to_be_retrieved			
			AMWebARSContainerDescriptor to_be_retrieved = (AMWebARSContainerDescriptor) container_descriptors.next();
			
			/* ***************************************************** *
			 * BEGIN: Protocol specific implementation...
			 * 
			 * Look up the AMWebARSContainer type: to_be_retrieved, 
			 * and store it in AMWebARSContainer type: retrieved_container.
			 * 
			 * This may involve sending a request to a third party profiling service
			 * or simply looking up the container in some sort of database.
			 * 
			 * For this example, we just one test container that was read from 
			 * a file called exampleContainer.xml during initialization.  So we 
			 * return that container for every valid container requested.
			 * 
			 * If a protocol is extended to return new containers, those 
			 * containers must be properly described in the 
			 * ContainerDescriptorTable.xml file (See the AMWebARS documentation
			 * for details about the Container Descriptor Table).
			 * ***************************************************** *
			 */
			
			try {
				// Get the Child Element called "example_protocol_test_container" from
				// the Root Element of _xml_test_data, and put it in a new AMWebARSContainer
				AMWebARSContainer retrieved_container = new AMWebARSContainer(_xml_test_data.getRootElement().getChild(TEST_CONTAINER_ID));		
				
				// If the retrieved container is valid...
				if (retrieved_container != null) {
					// Add it to the Client, and remove it from the hashtable of container
					// descriptors that must be retrieved.
					getClient().addContainer(retrieved_container);
					container_descriptors.remove();
				}
			}
			catch (MalformedInputException mal_ex) {
				return new ProtocolRunException(mal_ex);
			}
			
			/* ***************************************************** *
			 * END: Protocol specific implementation...
			 * ***************************************************** *
			 */
		}
		
		return PROTOCOL_RUN_SUCCESS;
	}
	
	/**
	 * @see amwebarsentitlementservice.protocol.AMWebARSProtocol#shutdown()
	 */
	public ProtocolShutdownStatus shutdown() {
		/* ***************************************************** *
		 * BEGIN: Protocol specific implementation...
		 * 
		 * This may involve disconnecting from a Third Party
		 * Profiling Service or local database.
		 * 
		 * For this example we have no connections, and therefore
		 * only have to set _initialized to false.
		 * 
		 * Also, in this example, no Exceptions can be thrown.  
		 * However, if code is written in this method that could 
		 * throw Exceptions, they should be handled by catching and 
		 * inserting them into ProtocolShutdownExceptions that would 
		 * then be returned.
		 * 
		 * Example:
		 * try {
		 *    // Perform Shutdown Processing
		 * } catch (Exception ex) {
		 *    ex.printStackTrace();
		 *    return new ProtocolShutdownException(ex);
		 * }
		 * ***************************************************** *
		 */
		 
		_initialized = false;
		return PROTOCOL_SHUTDOWN_SUCCESS;
		
		/* ***************************************************** *
		 * END: Protocol specific implementation...
		 * ***************************************************** *
		 */
	}
}
