#ifndef CDMF_UTILS_H_SOURCECODEID
#define CDMF_UTILS_H_SOURCECODEID
static char cdmf_utils_h_sourceCodeID[]="$Id: @(#)96  1.2.2.1 src/pdwebrte/cdsso/cdmf_utils.h, pdweb.cdsso, amweb510, 031011a 03/09/25 13:06:43 $";
#endif

/*                       
 *
 * Component Name: pdweb.cdsso
 * File Name: cdmf_utils.h
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2000,2001
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * All Rights Reserved.
 */ 

#ifndef _CDMF_UTILS_H
#define _CDMF_UTILS_H

#include <cdssotypes.h>

/* Function return codes */
#define CDMF_SUCCESS    (0)
#define CDMF_FAILURE    (1)
#define CDMF_NOMAP      (2)

#if defined(WIN32) || defined(_WIN32)
#if defined(CDMFUTILS_DLL)
#define CDMFUTILS_DECLSPEC       __declspec( dllexport )
#else
#define CDMFUTILS_DECLSPEC       __declspec( dllimport )
#endif
#define CDMFUTILS_CALLTYPE       __cdecl
#else /* WIN32 */
#define CDMFUTILS_DECLSPEC
#define CDMFUTILS_CALLTYPE
#endif /* WIN32 */

#ifdef __cplusplus
extern "C" 
{
#endif                         


/*
 * cdmf_create_usr_attr
 *
 * Create a new usr attribute. A copy is made of the name.
 *
 * ATTRIBUTES
 *
 * char * - [IN] Name of new attribute
 *
 * RETURNS - Pointer to newly allocated attribute on success, 
 *           NULL otherwise.
 */
CDMFUTILS_DECLSPEC
cdsso_usr_attr_t *
CDMFUTILS_CALLTYPE
cdmf_create_usr_attr(char *attr_name);


/*
 * cdmf_destroy_usr_attr
 *
 * Destroys a single usr attribute, including memory allocation of internals.
 *
 * ATTRIBUTES
 *
 * cdsso_usr_attr_t* attr - [IN] Attribute to destroy
 *
 * RETURNS - Nothing
 */
CDMFUTILS_DECLSPEC
void
CDMFUTILS_CALLTYPE
cdmf_destroy_usr_attr(cdsso_usr_attr_t *attr);


/*
 * cdmf_add_value_to_attr
 *
 * Add a new value to an attribute. A copy is made of the value.
 *
 * ARGUMENTS
 *
 * char             * - [IN] New value to be added to attribute
 * cdsso_usr_attr_t * - [OUT] Updated usr_attr obeject
 *
 * RETURNS - TRUE on success, 
 *           FALSE otherwise
 */
CDMFUTILS_DECLSPEC
int 
CDMFUTILS_CALLTYPE
cdmf_add_value_to_attr(char *new_value, cdsso_usr_attr_t *attr);


/*
 * cdmf_create_usr_attr_list
 *
 * Create an empty attribute list.
 *
 * ARGUMENTS
 *
 * None
 *
 * RETURNS - pointer to newly allocated list on success, 
 *           NULL otherwise.
 */
CDMFUTILS_DECLSPEC
cdsso_attrlist_t *
CDMFUTILS_CALLTYPE
cdmf_create_usr_attr_list(void);


/*
 * cdmf_destroy_usr_attr_list
 *
 * Destroys a usr attribute list, including memory allocation of internals.
 *
 * ATTRIBUTES
 *
 * cdsso_attrlist_t* list - [IN] Attribute list to destroy
 *
 * RETURNS - Nothing
 */
CDMFUTILS_DECLSPEC
void
CDMFUTILS_CALLTYPE
cdmf_destroy_usr_attr_list(cdsso_attrlist_t *list);


/*
 * cdmf_add_attr_to_list
 *
 * Add the specified attribute to the specified attribute list
 *
 * ARGUMENTS
 *
 * cdsso_usr_attr_t  * - [IN] New attribute to be added to the list
 * cdsso_attr_list_t * - [OUT] Updated list
 *
 * RETURNS - TRUE on success, 
 *           FALSE otherwise
 */
CDMFUTILS_DECLSPEC
int 
CDMFUTILS_CALLTYPE 
cdmf_add_attr_to_list(cdsso_usr_attr_t *new_attr, cdsso_attrlist_t *list);


/*
 * cdmf_get_num_attr_values
 *
 * ARGUMENTS
 *
 * cdsso_attrlist_t *list - [IN] Attribute list to search
 * const char* attr       - [IN] Name of attribute to search for
 * unsigned* n            - [OUT] Number of values for the named attribute
 *
 * RETURNS - Nothing
 */
CDMFUTILS_DECLSPEC
void
CDMFUTILS_CALLTYPE
cdmf_get_num_attr_values(cdsso_attrlist_t *list, const char* attr, unsigned* n);


/*
 * cdmf_get_attr_value
 *
 * ARGUMENTS
 *
 * cdsso_attrlist_t *list - [IN] Attribute list to search
 * const char* attr       - [IN] Name of attribute to search for
 * unsigned n             - [IN] Index of value to find
 * const char** value     - [OUT] n'th value of attribute attr in list or NULL if not present.
 *
 * RETURNS - Nothing
 */
CDMFUTILS_DECLSPEC
void
CDMFUTILS_CALLTYPE
cdmf_get_attr_value(cdsso_attrlist_t *list, const char* attr, unsigned n, const char** value);
    
/*
 * cdmf_replace_attr_value
 *
 * Replace a specified attribute value with a supplied one. The existing
 * value is removed and a copy made of the new one. If the attribute is
 * not found, or the value at the specified index is not found, the list
 * is not modified.
 *
 * ARGUMENTS
 *
 * const char* attr       - [IN] Name of attribute to replace value for
 * unsigned n             - [IN] Index value to find
 * const char* value      - [IN] New value for attribute, index.
 * cdsso_attrlist_t* list - [OUT] Updated list
 *
 * RETURNS - TRUE on success
 *           FALSE otherwise
 */
CDMFUTILS_DECLSPEC
int
CDMFUTILS_CALLTYPE
cdmf_replace_attr_value(const char* attr, unsigned n, const char* value, cdsso_attrlist_t* list);
    

#ifdef __cplusplus
}
#endif


#endif /* include guard */


