static char epac_c_sourceCodeID[]="$Id: @(#)21  1.1.2.1 src/pdwebrte/azn/epac_demo/epac.c, pdweb.webseal, amweb510, 040924a 04/08/26 18:58:01 $";

/* 
 * FILENAME
 *     epac.c
 *
 * DESCRIPTION
 *     This program takes a PAC from the HTTP_IV_CRED
 *     and converts it to a cred, printing out all the
 *     available information about it into a table.
 *
 *
 * Licensed Materials - Property of IBM
 * (C) Copyright International Business Machines Corp. 2001
 * All Rights Reserved
 *
 * ACE Team
 *
 */

/*-------------------------------------------------------------
 * Include list
 *-------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ogauthzn.h"
#include "aznutils.h"


/*-------------------------------------------------------------
 * Global definitions
 *-------------------------------------------------------------
 */
/*
 * Error checking macro for calls that *should* always work
 */
#define CHECK_AZN_STATUS(st,str)                      \
    if ( st != AZN_S_COMPLETE )                       \
    {                                                 \
        printf( "Call to %s failed: %d\n", str, st ); \
        printf( "Major: %d Minor: %d\n", azn_error_major( st ), azn_error_minor( st ) ); \
        azn_error_get_string( st, &errstr ); \
        printf( "Str: %s\n", errstr ); \
        abort();                                      \
    } 
#ifdef _WIN32
    #define TAM_BASE       "C:/Progra~1/Tivoli/Policy~1/"
    #define EXAMPLE_DIR    "example/authzn_demo/cpp/"
    #define TEMP_DIR       "C:/temp/"
#else
    #define TAM_BASE       "/opt/PolicyDirector/"
    #define EXAMPLE_DIR    "example/authzn_demo/cpp/"
    #define TEMP_DIR       "/tmp/"
#endif /* _WIN32 */


/*-------------------------------------------------------------
 * Global variables
 *-------------------------------------------------------------
 */
/*
 * Debug messages are written to this file
 */
const char  *debugfile           = TEMP_DIR"epac.log";
const char  *configfile          = TAM_BASE""EXAMPLE_DIR"/aznAPI.conf";
static       azn_string_t errstr = 0;



/*-------------------------------------------------------------
 * FUNCTION NAME
 *     debug_message
 *
 * DESCRIPTION
 *     This function is passed a message string that is to be
 *     used as debugging information when running the CGI.
 *     Once the string is identified, this function will
 *     create a file (specified as a global variable) which
 *     will contain this information. Additional debugging
 *     information will be appended to this file.
 *
 * ARGUMENTS - IN 
 *     message         String to be outputted to debug file.
 *
 * RETURNS 
 *     None
 *-------------------------------------------------------------
 */
void
debug_message(
               const char *message
             )
{
    FILE *df;

    df=fopen(debugfile, "a");
    if ( NULL == df ) {
        printf("Failed to open %s\n", debugfile);
        abort();
    }
    fprintf(df, "%s\n", message);
    if ( fclose(df) )
    {
        printf("Failed to close %s\n", debugfile);
        abort();
    }

} /* end of debug_message */



/*-------------------------------------------------------------
 * FUNCTION NAME
 *     print_formatted
 *
 * DESCRIPTION
 *     This function prints out the contents of the credential
 *     attribute list in a neatly formatted manner that will
 *     fit into the HTML table.
 *
 * ARGUMENTS - IN 
 *     s               String out of the cred attribute list. 
 *
 * RETURNS 
 *     None
 *-------------------------------------------------------------
 */
void
print_formatted( 
                 char *s
               )
{
    while( s && *s ) {
        if( *s == '<' ) {
            printf( "&lt;" );
        }
        else if( *s == '>' ) {
            printf( "&gt;" );
        }
        else {
            printf( "%c", *s );
        }

        s++;
    }

} /* end of print_formatted */



/*-------------------------------------------------------------
 * EPAC CGI Main
 *
 * Main program that runs the EPAC CGI. It gets the PAC from
 * the HTTP_IV_CRED and converts it to a cred using the aznapi
 * function calls and prints all the information out.
 *-------------------------------------------------------------
 */
main( int argc, char **argv )
{
    /*
     * Local variable declaration
     */
    char             *username  = NULL;
    char             *groups    = NULL;
    char             *epac      = NULL;
    char             *msg       = NULL;
    azn_string_t     *attr_names;
    azn_string_t      value;
    azn_creds_h_t     creds;
    azn_status_t      st;
    azn_attrlist_h_t  initdata, initinfo, creds_attrlist;
    azn_buffer_desc   pac;
    int               i, j;
    unsigned int      num_of_subjects, num_values;

    /*
     * Obtain environment variables
     */
    username = strdup(getenv("HTTP_IV_USER"));
    if ( NULL == username ) {
        printf("Failed to allocate memory for HTTP_IV_USER\n");
        abort();
    }
    groups = strdup(getenv("HTTP_IV_GROUPS"));
    if ( NULL == groups ) {
        printf("Failed to allocate memory for HTTP_IV_GROUPS\n");
        abort();
    }
    epac = strdup(getenv("HTTP_IV_CREDS"));
    if ( NULL == epac ) {
        printf("Failed to allocate memory for HTTP_IV_CREDS\n");
        abort();
    }

    /*
     * Print user's EPAC to debug file
     */
    debug_message( "Debug: User's EPAC" );

    /*
     * Strip off up to the space (only if found)
     */
    if( strchr( epac, ' ' ) != 0 ) {
        epac = strchr( epac, ' ' ) + 1;
    }
    debug_message(epac);

    /*
     * Create output HTML page
     */
    printf( "Content-type: text/html\n" );
    printf( "\n" );
    printf( "<HTML>\n" );
    printf( "<HEAD>\n" );
    printf( "<META http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n" );
    printf( "<TITLE>Tivoli Access Manager EPAC CGI</TITLE>\n" );
    printf( "</HEAD>\n" );
    printf( "<BODY text=White bgcolor=Black>\n" );
    printf( "<BR>\n" );
    printf( "username: <B>%s</B><BR>\n", username );
    printf( "groups: %s<BR>\n", groups );
    printf( "epac: %s<BR>\n", epac );

    debug_message( "Debug: header parsed" );

    /*
     * Initialize the attribute lists and check the status returned
     */
    st = azn_attrlist_create(&initdata);
    CHECK_AZN_STATUS( st, "azn_attrlist_create" );
    st = azn_attrlist_create(&initinfo);
    CHECK_AZN_STATUS( st, "azn_attrlist_create" );
    st = azn_attrlist_create(&creds_attrlist);
    CHECK_AZN_STATUS( st, "azn_attrlist_create" );

    debug_message( "Debug: attr lists initialized" );

    /*
     * Initialize the API from the config. file
     */
    if( st = azn_attrlist_add_entry(initdata, azn_init_cfg_file, (char *) configfile ) !=  AZN_S_COMPLETE ) {
        printf( "Error initialising API<BR>\n" );
        CHECK_AZN_STATUS( st, "azn_attrlist_add_entry" );
    }

    if ( (st = azn_initialize(initdata, &initinfo)) != AZN_S_COMPLETE) {
        printf("Error initialising API<BR>\n");
        CHECK_AZN_STATUS( st, "azn_initialize" );
    }

    /*
     * Call to azn_creds_create moved here AFTER the initialization
     */
    st = azn_creds_create( &creds );
    CHECK_AZN_STATUS( st, "azn_creds_create" );

    debug_message( "Debug: azn initialized" );

    pac.value = (unsigned char *) strdup(epac);
    if ( NULL == pac.value ) {
        printf( "Failed to allocate memory for epac's value.\n" );
        goto CLEANUP_AND_RETURN;
    }
    pac.length = strlen(epac);
    printf( "epac size: %d<BR>\n", pac.length );

    /*
     * Convert EPAC to cred
     */
    if( (st = azn_pac_get_creds(&pac, NULL, &creds)) != AZN_S_COMPLETE ) {
        printf( "Error converting EPAC: %d<BR>\n", st );
        goto CLEANUP_AND_RETURN;
    }

    if( (st = azn_creds_num_of_subjects(creds, &num_of_subjects)) != AZN_S_COMPLETE ) {
        printf( "Error getting number of creds in chain" );
        goto CLEANUP_AND_RETURN;
    }
    
        printf( "Number of creds: %d<br>\n", num_of_subjects );

    if( (st = azn_creds_get_attrlist_for_subject(creds, 0, &creds_attrlist)) != AZN_S_COMPLETE ) {
        printf( "Error getting extended fields" );
        goto CLEANUP_AND_RETURN;
    }

    if( (st = azn_attrlist_get_names(creds_attrlist, &attr_names)) != AZN_S_COMPLETE ) {
        printf( "Couldn't get names of extended fields" );
        goto CLEANUP_AND_RETURN;
    }

    if( attr_names == NULL ) {
        printf( "No extended fields found" );
        goto CLEANUP_AND_RETURN;
    }

    /*
     * Output the available information about the cred
     */
    printf( "<TABLE BORDER>\n" );
    printf( "<TR><B><TD>Index<TD>Name<TD>Value(s)</B></TR>\n" );

    i = 0;
    while ( attr_names[i] != 0 ) {
        printf( "<TR>" );
        printf( "<TD>%d<TD>%s<TD>", i, attr_names[i] );

        st = azn_attrlist_name_get_num( creds_attrlist,
                                        attr_names[i],
                                        &num_values
                                      );

        CHECK_AZN_STATUS( st, "azn_attrlist_name_get_num" );
        for ( j = 0; j < (int) num_values; j++ ) {
            st = azn_attrlist_get_entry_string_value( creds_attrlist,
                                                      attr_names[i],
                                                      j,
                                                      &value
                                                    );
            CHECK_AZN_STATUS( st, "azn_attrlist_get_entry_string_value" );
            printf( "[%d]: ", j );
            print_formatted( value );
            if( j < (int) (num_values-1) ) {
                printf( "<BR>" );
            }
        }

        printf("</TR>" );
        i++;
    }

    printf( "</TABLE>\n" );
    printf( "</BODY>\n" );
    printf( "</HTML>\n" );

    /*
     * Deliver any buffered data.
     */
    fflush(stdout);

    /*
     * Shutdown API
     */
    st = azn_shutdown();

    CLEANUP_AND_RETURN:
    /*
     * Free variables for which memory was allocated
     */
    if ( username ) {
        free( username );
        username = NULL;
    }
    if ( groups ) {
        free( groups );
        groups = NULL;
    }

} /* end of main */
