#ifndef PDXAUTHN_H_SOURCECODEID
#define PDXAUTHN_H_SOURCECODEID
static char pdxauthn_h_sourceCodeID[]="$Id: @(#)85  1.13.3.1 src/pdwebrte/authn/modules/pdxauthn/pdxauthnutils/pdxauthn.h, pdweb.authn, amweb510, 031011a 03/09/25 13:07:49 $";
#endif

/*
 * FILENAME
 *      pdxauthn.h
 *
 * DESCRIPTION
 *      Used in cross domain authentication module (CDAS), where client
 *      identity is generated base on the external authentication
 *      mechanism. PD converts this into PD's identity.
 *
 * COPYRIGHT NOTICE
 *      Copyright (c) 2001 Tivoli Systems.  All Rights Reserved.
 *
 */

#ifndef PDXAUTHN_H
#define PDXAUTHN_H

#if defined(__cplusplus)
extern "C" {
#endif

#if ! defined(pdnbase_v0_0_included)
typedef unsigned long int unsigned32;
#endif
typedef unsigned long xauthn_status_t;
typedef unsigned long xauthn_boolean_t;

#include "xattr.h"
#include "xnvlist.h"

#ifndef PDXAUTHNREF
extern void* __pdxauthn_referee;
static void** __pdxauthn_reference = &__pdxauthn_referee;
#endif

/*
 * Defined Constants
 */
#define XAUTHN_PRIN_TYPE_NAME 0
#define XAUTHN_PRIN_TYPE_DN   1
#define XAUTHN_PRIN_TYPE_URAF 2

#define XAUTHN_TAG_VALUE_KEY_PREFIX "tagvalue_"

/*
 * Possible identifier in the user authentication
 * name/value data list.
 */
#define XAUTHN_USERNAME       "xauthn_username"
#define XAUTHN_PASSWORD       "xauthn_password"
#define XAUTHN_NEW_PASSWORD   "xauthn_new_password"
#define XAUTHN_IPADDR         "xauthn_ipaddr"
#define XAUTHN_QOP            "xauthn_qop"
#define XAUTHN_BROWSER_INFO   "xauthn_browser_info"
#define XAUTHN_CERT           "xauthn_cert"
#define XAUTHN_CERT_DN        "xauthn_cert_dn"
#define XAUTHN_CERT_ISSUER_DN "xauthn_cert_issuer_dn"
#define XAUTHN_TOKEN          "xauthn_token"
#define XAUTHN_EXISTING_CRED  "xauthn_existing_cred"
#define XAUTHN_SU_METHOD      "xauthn_su_method"
#define XAUTHN_ADMIN_NAME     "xauthn_admin_name"
#define XAUTHN_ADMIN_CRED     "xauthn_admin_cred"
#define XAUTHN_FAILOVER_METHOD "xauthn_failover_method"
#define XAUTHN_NUM_ATTRS       "xauthn_num_attrs_"
#define XAUTHN_ATTR            "xauthn_attr_"
#define XAUTHN_INPUT_URL       "xauthn_input_url"
#define XAUTHN_VFT_STATUS      "xauthn_vft_status"
#define XAUTHN_TOKEN_TYPE      "xauthn_token_type"
#define XAUTHN_REDIRECT_URL    "xauthn_redirect_url"
#define XAUTHN_QUERY_STRING    "xauthn_query_string"
#define XAUTHN_SSO_TYPE        "xauthn_sso_type"
#define XAUTHN_METHOD          "xauthn_method"
#define XAUTHN_HOSTNAME	       "xauthn_hostname"
#define XAUTHN_REFERER         "xauthn_referer"


typedef struct xauthn_identity {
    struct {
        unsigned32 prin_type;
        union {
            /* case(s): 0 */
            char *name;
            /* case(s): 1 */
            char *dn;
            /* case(s): 2 */
            char *uraf_name;
        } data;
    } prin;
    char         *user_info;
    char         *authnmech_info;
    xattr_list_t xattrs;
} xauthn_identity_t;


/*
 * External interfaces that custom implementation is required.
 */
xauthn_status_t xauthn_initialize(int argc, const char **argv);
xauthn_status_t xauthn_authenticate(xnvlist_t *xauthnInfo, xauthn_identity_t *xauthnIdent);
xauthn_status_t xauthn_change_password(xnvlist_t *xauthnInfo);
xauthn_status_t xauthn_shutdown(int argc, const char **argv);



/*
 * Error status code
 */
#define XAUTHN_S_COMPLETE                     0x0000
#define XAUTHN_S_FAILURE                      0x0001
#define XAUTHN_S_INTERNAL_ERROR               0x0002
#define XAUTHN_S_UNKNOWN_ERROR                0x0003
#define XAUTHN_S_MECHANISM_ERROR              0x0004
#define XAUTHN_S_OUT_OF_MEMORY                0x0005
#define XAUTHN_S_OPERATION_ABORTED            0x0006
#define XAUTHN_S_UNKNOWN_IDENTITY_TYPE        0x0007
#define XAUTHN_S_CANT_LOAD_XATTRS             0x0008
#define XAUTHN_S_CANT_SELECT_MECHANISM        0x0009
#define XAUTHN_S_UNSUPPORTED_AUTHN_METHOD     0x000a
#define XAUTHN_S_MECHANISM_DOWN               0x000b
#define XAUTHN_S_INVALID_OPERATION            0x000c
#define XAUTHN_S_AUTHENTICATION_FAILURE       0x000d
#define XAUTHN_S_BAD_AUTHENTICATION_INFO      0x000e
#define XAUTHN_S_INVALID_USERNAME             0x000f
#define XAUTHN_S_RETRY_LIMIT_REACHED          0x0010
#define XAUTHN_S_PASSWORD_EXPIRED             0x0011
#define XAUTHN_S_ACCOUNT_EXPIRED              0x0012
#define XAUTHN_S_LOGIN_POLICY_VIOLATION       0x0013
#define XAUTHN_S_NEW_PIN                      0x0014
#define XAUTHN_S_ACCOUNT_DISABLED             0x0015
#define XAUTHN_S_NEXT_TOKEN                   0x0016
#define XAUTHN_S_MAPPING_FAILED               0x0017
#define XAUTHN_S_CERTIFICATE_AUTH_FAILURE     0x0018
#define XAUTHN_S_HTTP_HEADER_AUTH_FAILURE     0x0019
#define XAUTHN_S_IP_ADDR_AUTH_FAILED          0x001a
#define XAUTHN_S_SU_INVALID_USERNAME          0x001b
#define XAUTHN_S_SU_NO_PERMS                  0x001c
#define XAUTHN_S_PASSWD_POLICY_VIOLATION      0x001d
#define XAUTHN_S_PASSWD_TOO_SHORT             0x001e
#define XAUTHN_S_PASSWD_HAS_SPACES            0x001f
#define XAUTHN_S_PASSWD_TOO_MANY_REPEATED     0x0020
#define XAUTHN_S_PASSWD_TOO_FEW_ALPHAS        0x0021
#define XAUTHN_S_PASSWD_TOO_FEW_NONALPHAS     0x0022
#define XAUTHN_S_PASSWD_ACCT_LOCKEDOUT        0x0023
#define XAUTHN_S_POST_PWDCHG_FAILED           0x0024

#if defined(__cplusplus)
}
#endif

#endif /* end PDXAUTHN_H */
